#  BSD 3-Clause License
#
#  Copyright (c) 2012, the Sentry Team, see AUTHORS for more details
#  Copyright (c) 2019, Elasticsearch BV
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  * Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
#  * Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  * Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
#  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE


import datetime
import decimal
import uuid

try:
    import json
except ImportError:
    import simplejson as json


class BetterJSONEncoder(json.JSONEncoder):
    ENCODERS = {
        set: list,
        frozenset: list,
        datetime.datetime: lambda obj: obj.strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
        uuid.UUID: lambda obj: obj.hex,
        bytes: lambda obj: obj.decode("utf-8", errors="replace"),
        decimal.Decimal: lambda obj: float(obj),
    }

    def default(self, obj):
        if type(obj) in self.ENCODERS:
            return self.ENCODERS[type(obj)](obj)
        return super(BetterJSONEncoder, self).default(obj)


def better_decoder(data):
    return data


def dumps(value, **kwargs):
    return json.dumps(value, cls=BetterJSONEncoder, **kwargs)


def loads(value, **kwargs):
    return json.loads(value, object_hook=better_decoder)



#  BSD 3-Clause License
#
#  Copyright (c) 2012, the Sentry Team, see AUTHORS for more details
#  Copyright (c) 2019, Elasticsearch BV
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  * Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
#  * Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
#  * Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
#  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE


from django.apps import apps

from atatus.contrib.django.client import get_client
from atatus.middleware import Atatus as AtatusBase


class Atatus(AtatusBase):
    """
    Identical to the default WSGI middleware except that
    the client comes dynamically via ``get_client

    >>> from atatus.contrib.django.middleware.wsgi import Atatus
    >>> application = Atatus(application)
    """

    def __init__(self, application):
        self.application = application

    @property
    def client(self):
        try:
            app = apps.get_app_config("atatus.contrib.django")
            return app.client
        except LookupError:
            return get_client()




import json
import requests
from .builder import Builder
from atatus.utils import compat
from atatus.utils.logging import get_logger


class BaseTransport(object):
    def __init__(self, config, metadata):
        self._config = config
        self._error_logger = get_logger("atatus.errors")
        self._notify_host = config.notify_host if config.notify_host.endswith("/") else config.notify_host + "/"
        self._builder = Builder(config, metadata)
        self._session = requests.Session()
        self._blocked = False

    def _post(self, endpoint, data):
        if (self._blocked is True) and (endpoint != 'track/apm/hostinfo'):
            return

        try:
            r = self._session.post(self._notify_host + endpoint, timeout=30, json=data)

            if self._blocked is True:
                self._blocked = False

            if r.status_code != 200:
                self._error_logger.error(
                    "Atatus transport status non-200 response from backend [status_code: %r]" % r.status_code)

            if r.status_code == 400:
                c = r.content
                if not c:
                    self._error_logger.error("Atatus transport status 400, failed without content")
                    return

                if compat.PY3:
                    c = c.decode('UTF-8')

                resp = json.loads(c)
                if resp:
                    if "blocked" in resp:
                        self._blocked = resp["blocked"]
                        if self._blocked is True:
                            if "errorMessage" in resp:
                                self._error_logger.error(
                                    "Atatus blocked from sending data as: %s ", resp["errorMessage"])
                                return

                self._error_logger.error("Atatus transport status 400, failed with content: %r", c)
        except Exception as e:
            self._error_logger.error(
                "Atatus transport [%r] failed with exception: %r", self._notify_host + endpoint, e)
            raise

    def hostinfo(self, start_time):
        payload = self._builder.hostinfo(start_time)
        self._post('track/apm/hostinfo', payload)

    def txns(self, start_time, end_time, data):
        payload = self._builder.txns(start_time, end_time, data)
        self._post('track/apm/txn', payload)

    def traces(self, start_time, end_time, data):
        payload = self._builder.traces(start_time, end_time, data)
        self._post('track/apm/trace', payload)

    def error_metrics(self, start_time, end_time, metrics_data, requests_data):
        payload = self._builder.error_metrics(start_time, end_time, metrics_data, requests_data)
        self._post('track/apm/error_metric', payload)

    def errors(self, start_time, end_time, error_data):
        payload = self._builder.errors(start_time, end_time, error_data)
        self._post('track/apm/error', payload)

    def metrics(self, start_time, end_time, metrics_data):
        payload = self._builder.metrics(start_time, end_time, metrics_data)
        self._post('track/apm/metric', payload)
